@@ -9,32 +9,38 @@ import warnings
 from twisted import version as _txv
 
 # Declare top-level shortcuts
-from scrapy.spiders import Spider
-from scrapy.http import Request, FormRequest
+from scrapy.http import FormRequest, Request
+from scrapy.item import Field, Item
 from scrapy.selector import Selector
-from scrapy.item import Item, Field
-
+from scrapy.spiders import Spider
 
 __all__ = [
-    '__version__', 'version_info', 'twisted_version', 'Spider',
-    'Request', 'FormRequest', 'Selector', 'Item', 'Field',
+    "__version__",
+    "version_info",
+    "twisted_version",
+    "Spider",
+    "Request",
+    "FormRequest",
+    "Selector",
+    "Item",
+    "Field",
 ]
 
 
 # Scrapy and Twisted versions
 __version__ = (pkgutil.get_data(__package__, "VERSION") or b"").decode("ascii").strip()
-version_info = tuple(int(v) if v.isdigit() else v for v in __version__.split('.'))
+version_info = tuple(int(v) if v.isdigit() else v for v in __version__.split("."))
 twisted_version = (_txv.major, _txv.minor, _txv.micro)
 
 
 # Check minimum required Python version
-if sys.version_info < (3, 6):
-    print(f"Scrapy {__version__} requires Python 3.6+")
+if sys.version_info < (3, 8):
+    print(f"Scrapy {__version__} requires Python 3.8+")
     sys.exit(1)
 
 
 # Ignore noisy twisted deprecation warnings
-warnings.filterwarnings('ignore', category=DeprecationWarning, module='twisted')
+warnings.filterwarnings("ignore", category=DeprecationWarning, module="twisted")
 
 
 del pkgutil
