@@ -1,13 +1,19 @@
+from typing import Any
+
 from twisted.internet import defer
 from twisted.internet.base import ThreadedResolver
-from twisted.internet.interfaces import IHostResolution, IHostnameResolver, IResolutionReceiver, IResolverSimple
+from twisted.internet.interfaces import (
+    IHostnameResolver,
+    IHostResolution,
+    IResolutionReceiver,
+    IResolverSimple,
+)
 from zope.interface.declarations import implementer, provider
 
 from scrapy.utils.datatypes import LocalCache
 
-
 # TODO: cache misses
-dnscache = LocalCache(10000)
+dnscache: LocalCache[str, Any] = LocalCache(10000)
 
 
 @implementer(IResolverSimple)
@@ -23,16 +29,16 @@ class CachingThreadedResolver(ThreadedResolver):
 
     @classmethod
     def from_crawler(cls, crawler, reactor):
-        if crawler.settings.getbool('DNSCACHE_ENABLED'):
-            cache_size = crawler.settings.getint('DNSCACHE_SIZE')
+        if crawler.settings.getbool("DNSCACHE_ENABLED"):
+            cache_size = crawler.settings.getint("DNSCACHE_SIZE")
         else:
             cache_size = 0
-        return cls(reactor, cache_size, crawler.settings.getfloat('DNS_TIMEOUT'))
+        return cls(reactor, cache_size, crawler.settings.getfloat("DNS_TIMEOUT"))
 
     def install_on_reactor(self):
         self.reactor.installResolver(self)
 
-    def getHostByName(self, name, timeout=None):
+    def getHostByName(self, name: str, timeout=None):
         if name in dnscache:
             return defer.succeed(dnscache[name])
         # in Twisted<=16.6, getHostByName() is always called with
@@ -94,8 +100,8 @@ class CachingHostnameResolver:
 
     @classmethod
     def from_crawler(cls, crawler, reactor):
-        if crawler.settings.getbool('DNSCACHE_ENABLED'):
-            cache_size = crawler.settings.getint('DNSCACHE_SIZE')
+        if crawler.settings.getbool("DNSCACHE_ENABLED"):
+            cache_size = crawler.settings.getint("DNSCACHE_SIZE")
         else:
             cache_size = 0
         return cls(reactor, cache_size)
@@ -104,7 +110,12 @@ class CachingHostnameResolver:
         self.reactor.installNameResolver(self)
 
     def resolveHostName(
-        self, resolutionReceiver, hostName, portNumber=0, addressTypes=None, transportSemantics="TCP"
+        self,
+        resolutionReceiver,
+        hostName: str,
+        portNumber=0,
+        addressTypes=None,
+        transportSemantics="TCP",
     ):
         try:
             addresses = dnscache[hostName]
