@@ -7,24 +7,30 @@ import os
 from importlib import import_module
 from pathlib import Path
 from posixpath import split
-from unittest import mock
+from typing import Any, Coroutine, Dict, List, Optional, Tuple, Type
+from unittest import TestCase, mock
 
+from twisted.internet.defer import Deferred
 from twisted.trial.unittest import SkipTest
 
+from scrapy import Spider
+from scrapy.crawler import Crawler
 from scrapy.utils.boto import is_botocore_available
 
 
-def assert_gcs_environ():
+def assert_gcs_environ() -> None:
     if "GCS_PROJECT_ID" not in os.environ:
         raise SkipTest("GCS_PROJECT_ID not found")
 
 
-def skip_if_no_boto():
+def skip_if_no_boto() -> None:
     if not is_botocore_available():
         raise SkipTest("missing botocore library")
 
 
-def get_gcs_content_and_delete(bucket, path):
+def get_gcs_content_and_delete(
+    bucket: Any, path: str
+) -> Tuple[bytes, List[Dict[str, str]], Any]:
     from google.cloud import storage
 
     client = storage.Client(project=os.environ.get("GCS_PROJECT_ID"))
@@ -37,8 +43,13 @@ def get_gcs_content_and_delete(bucket, path):
 
 
 def get_ftp_content_and_delete(
-    path, host, port, username, password, use_active_mode=False
-):
+    path: str,
+    host: str,
+    port: int,
+    username: str,
+    password: str,
+    use_active_mode: bool = False,
+) -> bytes:
     from ftplib import FTP
 
     ftp = FTP()
@@ -46,33 +57,42 @@ def get_ftp_content_and_delete(
     ftp.login(username, password)
     if use_active_mode:
         ftp.set_pasv(False)
-    ftp_data = []
+    ftp_data: List[bytes] = []
 
-    def buffer_data(data):
+    def buffer_data(data: bytes) -> None:
         ftp_data.append(data)
 
     ftp.retrbinary(f"RETR {path}", buffer_data)
     dirname, filename = split(path)
     ftp.cwd(dirname)
     ftp.delete(filename)
-    return "".join(ftp_data)
+    return b"".join(ftp_data)
 
 
-def get_crawler(spidercls=None, settings_dict=None, prevent_warnings=True):
+class TestSpider(Spider):
+    name = "test"
+
+
+def get_crawler(
+    spidercls: Optional[Type[Spider]] = None,
+    settings_dict: Optional[Dict[str, Any]] = None,
+    prevent_warnings: bool = True,
+) -> Crawler:
     """Return an unconfigured Crawler object. If settings_dict is given, it
     will be used to populate the crawler settings with a project level
     priority.
     """
     from scrapy.crawler import CrawlerRunner
-    from scrapy.spiders import Spider
 
     # Set by default settings that prevent deprecation warnings.
-    settings = {}
+    settings: Dict[str, Any] = {}
     if prevent_warnings:
         settings["REQUEST_FINGERPRINTER_IMPLEMENTATION"] = "2.7"
     settings.update(settings_dict or {})
     runner = CrawlerRunner(settings)
-    return runner.create_crawler(spidercls or Spider)
+    crawler = runner.create_crawler(spidercls or TestSpider)
+    crawler._apply_settings()
+    return crawler
 
 
 def get_pythonpath() -> str:
@@ -82,7 +102,7 @@ def get_pythonpath() -> str:
     return str(Path(scrapy_path).parent) + os.pathsep + os.environ.get("PYTHONPATH", "")
 
 
-def get_testenv():
+def get_testenv() -> Dict[str, str]:
     """Return a OS environment dict suitable to fork processes that need to import
     this installation of Scrapy, instead of a system installed one.
     """
@@ -91,21 +111,23 @@ def get_testenv():
     return env
 
 
-def assert_samelines(testcase, text1, text2, msg=None):
+def assert_samelines(
+    testcase: TestCase, text1: str, text2: str, msg: Optional[str] = None
+) -> None:
     """Asserts text1 and text2 have the same lines, ignoring differences in
     line endings between platforms
     """
     testcase.assertEqual(text1.splitlines(), text2.splitlines(), msg)
 
 
-def get_from_asyncio_queue(value):
-    q = asyncio.Queue()
+def get_from_asyncio_queue(value: Any) -> Coroutine:
+    q: asyncio.Queue = asyncio.Queue()
     getter = q.get()
     q.put_nowait(value)
     return getter
 
 
-def mock_google_cloud_storage():
+def mock_google_cloud_storage() -> Tuple[Any, Any, Any]:
     """Creates autospec mocks for google-cloud-storage Client, Bucket and Blob
     classes and set their proper return values.
     """
@@ -122,7 +144,7 @@ def mock_google_cloud_storage():
     return (client_mock, bucket_mock, blob_mock)
 
 
-def get_web_client_agent_req(url):
+def get_web_client_agent_req(url: str) -> Deferred:
     from twisted.internet import reactor
     from twisted.web.client import Agent  # imports twisted.internet.reactor
 
