@@ -25,7 +25,6 @@ class MockServerSpider(Spider):
 
 
 class MetaSpider(MockServerSpider):
-
     name = "meta"
 
     def __init__(self, *args, **kwargs):
@@ -37,7 +36,6 @@ class MetaSpider(MockServerSpider):
 
 
 class FollowAllSpider(MetaSpider):
-
     name = "follow"
     link_extractor = LinkExtractor()
 
@@ -59,7 +57,6 @@ class FollowAllSpider(MetaSpider):
 
 
 class DelaySpider(MetaSpider):
-
     name = "delay"
 
     def __init__(self, n=1, b=0, *args, **kwargs):
@@ -80,8 +77,23 @@ class DelaySpider(MetaSpider):
         self.t2_err = time.time()
 
 
-class SimpleSpider(MetaSpider):
+class SlowSpider(DelaySpider):
+    name = "slow"
 
+    def start_requests(self):
+        # 1st response is fast
+        url = self.mockserver.url("/delay?n=0&b=0")
+        yield Request(url, callback=self.parse, errback=self.errback)
+
+        # 2nd response is slow
+        url = self.mockserver.url(f"/delay?n={self.n}&b={self.b}")
+        yield Request(url, callback=self.parse, errback=self.errback)
+
+    def parse(self, response):
+        yield Item()
+
+
+class SimpleSpider(MetaSpider):
     name = "simple"
 
     def __init__(self, url="http://localhost:8998", *args, **kwargs):
@@ -93,7 +105,6 @@ class SimpleSpider(MetaSpider):
 
 
 class AsyncDefSpider(SimpleSpider):
-
     name = "asyncdef"
 
     async def parse(self, response):
@@ -102,7 +113,6 @@ class AsyncDefSpider(SimpleSpider):
 
 
 class AsyncDefAsyncioSpider(SimpleSpider):
-
     name = "asyncdef_asyncio"
 
     async def parse(self, response):
@@ -112,7 +122,6 @@ class AsyncDefAsyncioSpider(SimpleSpider):
 
 
 class AsyncDefAsyncioReturnSpider(SimpleSpider):
-
     name = "asyncdef_asyncio_return"
 
     async def parse(self, response):
@@ -123,7 +132,6 @@ class AsyncDefAsyncioReturnSpider(SimpleSpider):
 
 
 class AsyncDefAsyncioReturnSingleElementSpider(SimpleSpider):
-
     name = "asyncdef_asyncio_return_single_element"
 
     async def parse(self, response):
@@ -134,7 +142,6 @@ class AsyncDefAsyncioReturnSingleElementSpider(SimpleSpider):
 
 
 class AsyncDefAsyncioReqsReturnSpider(SimpleSpider):
-
     name = "asyncdef_asyncio_reqs_return"
 
     async def parse(self, response):
@@ -191,7 +198,6 @@ class AsyncDefDeferredMaybeWrappedSpider(SimpleSpider):
 
 
 class AsyncDefAsyncioGenSpider(SimpleSpider):
-
     name = "asyncdef_asyncio_gen"
 
     async def parse(self, response):
@@ -201,7 +207,6 @@ class AsyncDefAsyncioGenSpider(SimpleSpider):
 
 
 class AsyncDefAsyncioGenLoopSpider(SimpleSpider):
-
     name = "asyncdef_asyncio_gen_loop"
 
     async def parse(self, response):
@@ -212,7 +217,6 @@ class AsyncDefAsyncioGenLoopSpider(SimpleSpider):
 
 
 class AsyncDefAsyncioGenComplexSpider(SimpleSpider):
-
     name = "asyncdef_asyncio_gen_complex"
     initial_reqs = 4
     following_reqs = 3
@@ -246,7 +250,6 @@ class AsyncDefAsyncioGenComplexSpider(SimpleSpider):
 
 
 class ItemSpider(FollowAllSpider):
-
     name = "item"
 
     def parse(self, response):
@@ -261,7 +264,6 @@ class DefaultError(Exception):
 
 
 class ErrorSpider(FollowAllSpider):
-
     name = "error"
     exception_cls = DefaultError
 
@@ -275,7 +277,6 @@ class ErrorSpider(FollowAllSpider):
 
 
 class BrokenStartRequestsSpider(FollowAllSpider):
-
     fail_before_yield = False
     fail_yielding = False
 
@@ -305,7 +306,6 @@ class BrokenStartRequestsSpider(FollowAllSpider):
 
 
 class SingleRequestSpider(MetaSpider):
-
     seed = None
     callback_func = None
     errback_func = None
@@ -451,7 +451,6 @@ class CrawlSpiderWithProcessRequestCallbackKeywordArguments(CrawlSpiderWithParse
 
 
 class BytesReceivedCallbackSpider(MetaSpider):
-
     full_response_length = 2**18
 
     @classmethod
