@@ -122,7 +122,7 @@ class RetryTest(unittest.TestCase):
         req = Request(f"http://www.scrapytest.org/{exc.__name__}")
         self._test_retry_exception(req, exc("foo"), mw)
 
-    def test_exception_to_retry_customMiddleware(self):
+    def test_exception_to_retry_custom_middleware(self):
         exc = ValueError
 
         with warnings.catch_warnings(record=True) as warns:
@@ -138,6 +138,21 @@ class RetryTest(unittest.TestCase):
         assert isinstance(req, Request)
         self.assertEqual(req.meta["retry_times"], 1)
 
+    def test_exception_to_retry_custom_middleware_self(self):
+        class MyRetryMiddleware(RetryMiddleware):
+            def process_exception(self, request, exception, spider):
+                if isinstance(exception, self.EXCEPTIONS_TO_RETRY):
+                    return self._retry(request, exception, spider)
+
+        exc = OSError
+        mw2 = MyRetryMiddleware.from_crawler(self.crawler)
+        req = Request(f"http://www.scrapytest.org/{exc.__name__}")
+        with warnings.catch_warnings(record=True) as warns:
+            req = mw2.process_exception(req, exc("foo"), self.spider)
+        assert isinstance(req, Request)
+        self.assertEqual(req.meta["retry_times"], 1)
+        self.assertEqual(len(warns), 1)
+
     def _test_retry_exception(self, req, exception, mw=None):
         if mw is None:
             mw = self.mw
